@@ -7,12 +7,6 @@ import os
 import sys
 import locale
 
-def _scriptName():
-    return os.path.basename(sys.argv[0]).replace('.py','')
-
-def _encoding():
-    return locale.getpreferredencoding().lower()
-
 class _Config(object):
     """
     Contains configuration parameters for the dogtail run.
@@ -28,7 +22,7 @@ class _Config(object):
 
     scriptName(str) [Read-Only]:
     The name of the script being run.
-    
+
     encoding(str)
     The encoding for text, used by dogtail.tc.TCString .
 
@@ -39,13 +33,13 @@ class _Config(object):
     The delay after a character is typed on the keyboard.
 
     runInterval(float):
-    The interval at which dogtail.utils.run() and dogtail.procedural.run() 
+    The interval at which dogtail.utils.run() and dogtail.procedural.run()
     check to see if the application has started up.
 
     runTimeout(int):
     The timeout after which dogtail.utils.run() and dogtail.procedural.run()
     give up on looking for the newly-started application.
-    
+
     searchBackoffDuration (float):
     Time in seconds for which to delay when a search fails.
 
@@ -58,6 +52,10 @@ class _Config(object):
     defaultDelay (float):
     Default time in seconds to sleep when delaying.
 
+    childrenLimit (int):
+    When there are a very large number of children of a node, only return
+    this many, starting with the first.
+
     debugSearching (boolean):
     Whether to write info on search backoff and retry to the debug log.
 
@@ -88,30 +86,38 @@ class _Config(object):
 
     fatalErrors (boolean):
     Whether errors encountered in dogtail.procedural should be considered
-    fatal. If True, exceptions will be raised. If False, warnings will be 
+    fatal. If True, exceptions will be raised. If False, warnings will be
     passed to the debug logger.
 
-    useIconLogger (boolean):
-    Whether we should place an icon in the notification area and print debug
-    messages to it.
+    checkForA11y (boolean):
+    Whether to check if accessibility is enabled. If not, just assume it is
+    (default True).
 
     logDebugToFile (boolean):
     Whether to write debug output to a log file.
+
+    logDebugToStdOut (boolean):
+    Whether to print log output to console or not (default True).
     """
-    __scriptName = staticmethod(_scriptName)
-    __encoding = staticmethod(_encoding)
+    def _getScriptName(self):
+        return os.path.basename(sys.argv[0]).replace('.py','')
+    scriptName = property(_getScriptName)
+
+    def _getEncoding(self):
+        return locale.getpreferredencoding().lower()
+    encoding = property(_getEncoding)
 
     defaults = {
             # Storage
             'scratchDir' : '/tmp/dogtail/',
             'dataDir' : '/tmp/dogtail/data/',
             'logDir' : '/tmp/dogtail/logs/',
-            'scriptName' : _scriptName(),
-            'encoding' : _encoding(),
+            'scriptName' : scriptName.fget(None),
+            'encoding' : encoding.fget(None),
             'configFile' : None,
             'baseFile' : None,
 
-            # Timing
+            # Timing and Limits
             'actionDelay' : 1.0,
             'typingDelay' : 0.075,
             'runInterval' : 0.5,
@@ -120,19 +126,21 @@ class _Config(object):
             'searchWarningThreshold' : 3,
             'searchCutoffCount' : 20,
             'defaultDelay' : 0.5,
+            'childrenLimit' : 100,
 
             # Debug
             'debugSearching' : False,
             'debugSleep' : False,
             'debugSearchPaths' : False,
+            'logDebugToStdOut' : True,
             'absoluteNodePaths' : False,
             'ensureSensitivity' : False,
             'debugTranslation' : False,
             'blinkOnActions' : False,
             'fatalErrors' : False,
+            'checkForA11y' : True,
 
             # Logging
-            'useIconLogger' : False,
             'logDebugToFile' : True
     }
 
@@ -141,58 +149,46 @@ class _Config(object):
     invalidValue = "__INVALID__"
 
     def __init__(self):
-        scriptName = _scriptName()
-        encoding = _encoding()
         _Config.__createDir(_Config.defaults['scratchDir'])
         _Config.__createDir(_Config.defaults['logDir'])
         _Config.__createDir(_Config.defaults['dataDir'])
 
     def __setattr__(self, name, value):
-        if not config.defaults.has_key(name):
+        if name not in config.defaults:
             raise AttributeError, name + " is not a valid option."
 
         elif _Config.defaults[name] != value or \
                 _Config.options.get(name, _Config.invalidValue) != value:
             if 'Dir' in name:
                 _Config.__createDir(value)
-                if value[-1] != '/': value = value + '/'
-            elif name == 'useIconLogger':
+                if value[-1] != os.path.sep: value = value + os.path.sep
+            elif name == 'logDebugToFile':
                 import logging
-                if value == True:
-                    logging.Logger.iconLogger = logging.IconLogger()
-                elif value == False:
-                    del logging.Logger.iconLogger
-                    logging.Logger.iconLogger = None
-                else: raise ValueError, value
+                logging.debugLogger = logging.Logger('debug', value)
             _Config.options[name] = value
 
     def __getattr__(self, name):
         try: return _Config.options[name]
         except KeyError:
             try: return _Config.defaults[name]
-            except KeyError: raise AttributeError, name + " is not a valid option."
+            except KeyError: raise AttributeError, name + \
+                    " is not a valid option."
 
-    def __createDir(cls, dirName):
+    def __createDir(cls, dirName, perms = 0777):
         """
-        Creates a directory (if it doesn't currently exist), creating any parent directories it needs.
+        Creates a directory (if it doesn't currently exist), creating any
+        parent directories it needs.
+
+        If perms is None, create with python's default permissions.
         """
         dirName = os.path.abspath(dirName)
         #print "Checking for %s ..." % dirName,
         if not os.path.isdir(dirName):
-            #print "Not found."
-            parentDirName = os.path.sep + os.path.sep.join(os.path.split(dirName + os.path.sep)[0].split(os.path.sep)[1:-1])
-            #print "Checking for parent %s ..." % parentDirName,
-            if not os.path.isdir(parentDirName):
-                #print "Not found."
-                #print "Parent %s ..." % parentDirName
-                _Config.__createDir(parentDirName)
-                print "Creating %s ..." % dirName
-                os.mkdir(dirName)
-            else:
-                #print "Found."
-                print "Creating %s ..." % dirName
-                os.mkdir(dirName)
-        #else: print "Found."
+            if perms:
+                umask = os.umask(0)
+                os.makedirs(dirName, perms)
+                os.umask(umask)
+            else: os.makedirs(dirName)
     __createDir = classmethod(__createDir)
 
     def load(self, dict):
@@ -220,10 +216,11 @@ if __name__ == '__main__':
 
     # BEGIN tests
 
-    failure = True
+    failure = False
     for option in config.defaults.keys():
-        #print failure, getattr(config, option), config.defaults[option]
-        failure = not failure and ( getattr(config, option) == config.defaults[option])
+        failure = failure or not (getattr(config, option) == \
+                config.defaults[option])
+        print failure, option, getattr(config, option), config.defaults[option]
     failOrPass(failure, "Reading all default values")
 
     failure = True
@@ -249,15 +246,6 @@ if __name__ == '__main__':
     failure = failure or not os.path.isdir('/tmp/dt_data')
     failOrPass(failure, "Changing default directories")
 
-    failure = True
-    config.useIconLogger = True
-    import logging
-    try:
-        if isinstance(logging.debugLogger.iconLogger, logging.IconLogger):
-            failure = False
-    except AttributeError: pass
-    failOrPass(failure, "Setting useIconLogger")
-
     # END tests
 
     if anyFailed: sys.exit(1)
