@@ -16,18 +16,18 @@ __author__ = 'Zack Cerza <zcerza@redhat.com>'
 ##############################################################################
 
 import tree
-from predicate import GenericPredicate, IsADialogNamed, IsAnApplicationNamed
+import predicate
 from config import config
+from utils import Lock
 import rawinput
-from time import sleep
 
 #FocusError = "FocusError: %s not found"
 class FocusError(Exception):
     pass
 
 import errors
-def focusFailed():
-    errors.warn('The requested object was not found')
+def focusFailed(pred):
+    errors.warn('The requested widget could not be focused: %s' % pred.debugName)
 
 ENOARGS = "At least one argument is needed"
 
@@ -40,16 +40,16 @@ class FocusBase:
     def __getattr__ (self, name):
         # Fold all the Node's AT-SPI properties into the Focus object.
         try: return getattr(self.node, name)
-        except AttributeError: 
+        except AttributeError:
             raise AttributeError, name
 
     def __setattr__ (self, name, value):
         # Fold all the Node's AT-SPI properties into the Focus object.
         if name == 'node':
-            self.__dict__[name] = value
+            self.__class__.__dict__[name] = value
         else:
             try: setattr(self.node, name, value)
-            except AttributeError: 
+            except AttributeError:
                 raise AttributeError, name
 
 class FocusApplication (FocusBase):
@@ -62,17 +62,18 @@ class FocusApplication (FocusBase):
         Search for an application that matches and refocus on the given name.
         """
         try:
-            predicate = IsAnApplicationNamed(name)
-            app = self.desktop.findChild(predicate, recursive = False, retry = False)
+            pred = predicate.IsAnApplicationNamed(name)
+            app = self.desktop.findChild(pred, recursive = False, retry = False)
         except tree.SearchError, desc:
             if config.fatalErrors: raise FocusError, name
             else:
-                focusFailed()
-                return
-        if app: 
+                focusFailed(pred)
+                return False
+        if app:
             FocusApplication.node = app
             FocusDialog.node = None
             FocusWidget.node = None
+        return True
 
 class FocusDesktop (FocusBase):
     """
@@ -80,6 +81,30 @@ class FocusDesktop (FocusBase):
     """
     pass
 
+class FocusWindow (FocusBase):
+    """
+    Keeps track of which window is currently focused.
+    """
+    def __call__ (self, name):
+        """
+        Search for a dialog that matches the given name and refocus on it.
+        """
+        result = None
+        pred = predicate.IsAWindowNamed(name)
+        try:
+            result = FocusApplication.node.findChild(pred, requireResult=False, recursive = False)
+        except AttributeError: pass
+        if result:
+            FocusWindow.node = result
+            FocusDialog.node = None
+            FocusWidget.node = None
+        else:
+            if config.fatalErrors: raise FocusError, pred.debugName
+            else:
+                focusFailed(pred)
+                return False
+        return True
+
 class FocusDialog (FocusBase):
     """
     Keeps track of which dialog is currently focused.
@@ -89,23 +114,58 @@ class FocusDialog (FocusBase):
         Search for a dialog that matches the given name and refocus on it.
         """
         result = None
-        predicate = IsADialogNamed(name)
+        pred = predicate.IsADialogNamed(name)
         try:
-            result = FocusApplication.node.findChild(predicate, requireResult=False, recursive = False)
+            result = FocusApplication.node.findChild(pred, requireResult=False, recursive = False)
         except AttributeError: pass
         if result:
             FocusDialog.node = result
             FocusWidget.node = None
-        else: 
-            if config.fatalErrors: raise FocusError, predicate.debugName
+        else:
+            if config.fatalErrors: raise FocusError, pred.debugName
             else:
-                focusFailed()
-                return
+                focusFailed(pred)
+                return False
+        return True
 
 class FocusWidget (FocusBase):
     """
     Keeps track of which widget is currently focused.
     """
+    def findByPredicate(self, pred):
+        result = None
+        try:
+            result = FocusWidget.node.findChild(pred, requireResult = False, retry = False)
+        except AttributeError: pass
+        if result: FocusWidget.node = result
+        else:
+            try:
+                result = FocusDialog.node.findChild(pred, requireResult = False, retry = False)
+            except AttributeError: pass
+        if result: FocusWidget.node = result
+        else:
+            try:
+                result = FocusWindow.node.findChild(pred, requireResult = False, retry = False)
+            except AttributeError: pass
+        if result: FocusWidget.node = result
+        else:
+            try:
+                result = FocusApplication.node.findChild(pred, requireResult = False, retry = False)
+                if result: FocusWidget.node = result
+            except AttributeError:
+                if config.fatalErrors: raise FocusError, name
+                else:
+                    focusFailed(pred)
+                    return False
+
+        if result == None:
+            FocusWidget.node = result
+            if config.fatalErrors: raise FocusError, pred.debugName
+            else:
+                focusFailed(pred)
+                return False
+        return True
+
     def __call__ (self, name = '', roleName = '', description = ''):
         """
         If name, roleName or description are specified, search for a widget that matches and refocus on it.
@@ -114,33 +174,9 @@ class FocusWidget (FocusBase):
             raise TypeError, ENOARGS
 
         # search for a widget.
-        result = None
-        predicate = GenericPredicate(name = name, roleName = roleName, description = description)
-        try:
-            result = FocusWidget.node.findChild(predicate, requireResult = False, retry = False)
-        except AttributeError: pass
-        if result: FocusWidget.node = result
-        else:
-            try:
-                result = FocusDialog.node.findChild(predicate, requireResult = False, retry = False)
-            except AttributeError: pass
-        if result: FocusWidget.node = result
-        else:
-            try:
-                result = FocusApplication.node.findChild(predicate, requireResult = False, retry = False)
-                if result: FocusWidget.node = result
-            except AttributeError: 
-                if config.fatalErrors: raise FocusError, name
-                else:
-                    focusFailed()
-                    return
-
-        if result == None:
-            FocusWidget.node = result
-            if config.fatalErrors: raise FocusError, predicate.debugName
-            else:
-                focusFailed()
-                return
+        pred = predicate.GenericPredicate(name = name,
+                roleName = roleName, description = description)
+        return self.findByPredicate(pred)
 
 class Focus (FocusBase):
     """
@@ -150,7 +186,7 @@ class Focus (FocusBase):
     def __getattr__ (self, name):
         raise AttributeError, name
     def __setattr__(self, name, value):
-        if name in ('application', 'dialog', 'widget'):
+        if name in ('application', 'dialog', 'widget', 'window'):
             self.__dict__[name] = value
         else:
             raise AttributeError, name
@@ -159,61 +195,51 @@ class Focus (FocusBase):
     application = FocusApplication()
     app = application # shortcut :)
     dialog = FocusDialog()
+    window = FocusWindow()
+    frame = window
     widget = FocusWidget()
 
     def button (self, name):
         """
-        A shortcut to self.widget(name, roleName = 'push button')
+        A shortcut to self.widget.findByPredicate(predicate.IsAButtonNamed(name))
         """
-        self.widget(name = name, roleName = 'push button')
-
-    def frame (self, name):
-        """
-        A shortcut to self.widget(name, roleName = 'frame')
-        """
-        self.widget(name = name, roleName = 'frame')
+        return self.widget.findByPredicate(predicate.IsAButtonNamed(name))
 
     def icon (self, name):
         """
         A shortcut to self.widget(name, roleName = 'icon')
         """
-        self.widget(name = name, roleName = 'icon')
+        return self.widget(name = name, roleName = 'icon')
 
     def menu (self, name):
         """
-        A shortcut to self.widget(name, roleName = 'menu')
+        A shortcut to self.widget.findByPredicate(predicate.IsAMenuNamed(name))
         """
-        self.widget(name = name, roleName = 'menu')
+        return self.widget.findByPredicate(predicate.IsAMenuNamed(name))
 
     def menuItem (self, name):
         """
-        A shortcut to self.widget(name, roleName = 'menu item')
+        A shortcut to self.widget.findByPredicate(predicate.IsAMenuItemNamed(name))
         """
-        self.widget(name = name, roleName = 'menu item')
+        return self.widget.findByPredicate(predicate.IsAMenuItemNamed(name))
 
     def table (self, name = ''):
         """
         A shortcut to self.widget(name, roleName 'table')
         """
-        self.widget(name = name, roleName = 'table')
+        return self.widget(name = name, roleName = 'table')
 
     def tableCell (self, name = ''):
         """
         A shortcut to self.widget(name, roleName 'table cell')
         """
-        self.widget(name = name, roleName = 'table cell')
+        return self.widget(name = name, roleName = 'table cell')
 
     def text (self, name = ''):
         """
-        A shortcut to self.widget(name, roleName = 'text')
+        A shortcut to self.widget.findByPredicate(IsATextEntryNamed(name))
         """
-        self.widget(name = name, roleName = 'text')
-
-    def window (self, name):
-        """
-        A shortcut to self.widget(name, roleName = 'window')
-        """
-        self.widget(name = name, roleName = 'window')
+        return self.widget.findByPredicate(predicate.IsATextEntryNamed(name))
 
 class Action (FocusWidget):
     """
@@ -232,8 +258,7 @@ class Action (FocusWidget):
         """
         if name or roleName or description:
             FocusWidget.__call__(self, name = name, roleName = roleName, description = description)
-        self.node.doAction(self.action)
-        sleep(delay)
+        self.node.doActionNamed(self.action)
 
     def __getattr__ (self, attr):
         return getattr(FocusWidget.node, attr)
@@ -289,15 +314,17 @@ class Click (Action):
     def __init__ (self):
         Action.__init__(self, 'click')
 
-    def __call__ (self, name = '', roleName = '', description = '', raw = False, button = primary, delay = config.actionDelay):
+    def __call__ (self, name = '', roleName = '', description = '', raw = True, button = primary, delay = config.actionDelay):
         """
-        If coords or button are specified, execute a raw mouse event. If not, just pass the rest of the arguments to Action.
+        By default, execute a raw mouse event.
+        If raw is False or if button evaluates to False, just pass the rest of
+        the arguments to Action.
         """
+        if name or roleName or description:
+            FocusWidget.__call__(self, name = name, roleName = roleName, description = description)
         if raw and button:
             # We're doing a raw mouse click
-            FocusWidget.__call__(self, name = name, roleName = roleName, description = description)
-            Click.node.rawClick(button)
-            sleep(delay)
+            Click.node.click(button)
         else:
             Action.__call__(self, name = name, roleName = roleName, description = description, delay = delay)
 
@@ -324,8 +351,7 @@ class Select (Action):
             FocusWidget.__call__(self, name = name, roleName = roleName, description = description)
         func = getattr(self.node, self.action)
         func()
-        sleep(delay)
-        
+
 def type(text):
     if focus.widget.node:
         focus.widget.node.typeText(text)
@@ -340,8 +366,17 @@ def keyCombo(combo):
 
 def run(application, arguments = '', appName = ''):
     from utils import run as utilsRun
-    utilsRun(application + arguments, appName = appName)
+    pid = utilsRun(application + ' ' + arguments, appName = appName)
     focus.application(application)
+    return pid
+
+# tell sniff not to use auto-refresh while script using this module is running
+sniff_lock = Lock(lockname='sniff_refresh.lock',randomize=False)
+try:
+    sniff_lock.lock()
+except OSError:
+    pass # lock was already present from other script instance or leftover from killed instance
+# lock should unlock automatically on script exit.
 
 focus = Focus()
 click = Click()
